@@ -1,8 +1,8 @@
 /*
       Objects which encapsulate discretizations+continuum residuals
 */
-#if !defined(__PETSCCE_H)
-#define __PETSCCE_H
+#if !defined(PETSCCE_H)
+#define PETSCCE_H
 #include <petscsnes.h>
 
 /*S
@@ -18,10 +18,13 @@ PETSC_EXTERN PetscErrorCode PetscConvEstCreate(MPI_Comm, PetscConvEst *);
 PETSC_EXTERN PetscErrorCode PetscConvEstDestroy(PetscConvEst *);
 PETSC_EXTERN PetscErrorCode PetscConvEstView(PetscConvEst, PetscViewer);
 PETSC_EXTERN PetscErrorCode PetscConvEstSetFromOptions(PetscConvEst);
-PETSC_EXTERN PetscErrorCode PetscConvEstGetSolver(PetscConvEst, SNES *);
-PETSC_EXTERN PetscErrorCode PetscConvEstSetSolver(PetscConvEst, SNES);
+PETSC_EXTERN PetscErrorCode PetscConvEstGetSolver(PetscConvEst, PetscObject *);
+PETSC_EXTERN PetscErrorCode PetscConvEstSetSolver(PetscConvEst, PetscObject);
 PETSC_EXTERN PetscErrorCode PetscConvEstSetUp(PetscConvEst);
+PETSC_EXTERN PetscErrorCode PetscConvEstComputeInitialGuess(PetscConvEst, PetscInt, DM, Vec);
+PETSC_EXTERN PetscErrorCode PetscConvEstComputeError(PetscConvEst, PetscInt, DM, Vec, PetscReal[]);
 PETSC_EXTERN PetscErrorCode PetscConvEstGetConvRate(PetscConvEst, PetscReal[]);
-PETSC_EXTERN PetscErrorCode PetscConvEstRateView(PetscConvEst, PetscReal[], PetscViewer);
+PETSC_EXTERN PetscErrorCode PetscConvEstMonitorDefault(PetscConvEst, PetscInt);
+PETSC_EXTERN PetscErrorCode PetscConvEstRateView(PetscConvEst, const PetscReal[], PetscViewer);
 
 #endif
