@@ -1,7 +1,7 @@
 
 #include <petsc/private/matimpl.h>        /*I "petscmat.h" I*/
 
-const char *const MatCompositeMergeTypes[] = {"left","right","MatCompositeMergeType","MAT_COMPOSITE_",0};
+const char *const MatCompositeMergeTypes[] = {"left","right","MatCompositeMergeType","MAT_COMPOSITE_",NULL};
 
 typedef struct _Mat_CompositeLink *Mat_CompositeLink;
 struct _Mat_CompositeLink {
@@ -535,7 +535,7 @@ static PetscErrorCode MatCompositeAddMat_Composite(Mat mat,Mat smat)
 
   PetscFunctionBegin;
   ierr        = PetscNewLog(mat,&ilink);CHKERRQ(ierr);
-  ilink->next = 0;
+  ilink->next = NULL;
   ierr        = PetscObjectReference((PetscObject)smat);CHKERRQ(ierr);
   ilink->mat  = smat;
 
@@ -589,7 +589,7 @@ static PetscErrorCode MatCompositeSetType_Composite(Mat mat,MatCompositeType typ
   PetscFunctionBegin;
   b->type = type;
   if (type == MAT_COMPOSITE_MULTIPLICATIVE) {
-    mat->ops->getdiagonal   = 0;
+    mat->ops->getdiagonal   = NULL;
     mat->ops->mult          = MatMult_Composite_Multiplicative;
     mat->ops->multtranspose = MatMultTranspose_Composite_Multiplicative;
     b->merge_mvctx          = PETSC_FALSE;
@@ -988,148 +988,148 @@ PetscErrorCode MatCompositeSetScalings(Mat mat,const PetscScalar *scalings)
   PetscFunctionReturn(0);
 }
 
-static struct _MatOps MatOps_Values = {0,
-                                       0,
-                                       0,
+static struct _MatOps MatOps_Values = {NULL,
+                                       NULL,
+                                       NULL,
                                        MatMult_Composite,
                                        MatMultAdd_Composite,
-                                /*  5*/ MatMultTranspose_Composite,
+                               /*  5*/ MatMultTranspose_Composite,
                                        MatMultTransposeAdd_Composite,
-                                       0,
-                                       0,
-                                       0,
-                                /* 10*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                                /* 15*/ 0,
-                                       0,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /* 10*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /* 15*/ NULL,
+                                       NULL,
                                        MatGetDiagonal_Composite,
                                        MatDiagonalScale_Composite,
-                                       0,
-                                /* 20*/ 0,
+                                       NULL,
+                               /* 20*/ NULL,
                                        MatAssemblyEnd_Composite,
-                                       0,
-                                       0,
-                               /* 24*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /* 29*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /* 34*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /* 39*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /* 44*/ 0,
+                                       NULL,
+                                       NULL,
+                               /* 24*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /* 29*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /* 34*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /* 39*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /* 44*/ NULL,
                                        MatScale_Composite,
                                        MatShift_Basic,
-                                       0,
-                                       0,
-                               /* 49*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /* 54*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /* 59*/ 0,
+                                       NULL,
+                                       NULL,
+                               /* 49*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /* 54*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /* 59*/ NULL,
                                        MatDestroy_Composite,
-                                       0,
-                                       0,
-                                       0,
-                               /* 64*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /* 69*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /* 74*/ 0,
-                                       0,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /* 64*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /* 69*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /* 74*/ NULL,
+                                       NULL,
                                        MatSetFromOptions_Composite,
-                                       0,
-                                       0,
-                               /* 79*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /* 84*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /* 89*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /* 94*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                                /*99*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /*104*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /*109*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /*114*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /*119*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /*124*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /*129*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /*134*/ 0,
-                                       0,
-                                       0,
-                                       0,
-                                       0,
-                               /*139*/ 0,
-                                       0,
-                                       0
+                                       NULL,
+                                       NULL,
+                               /* 79*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /* 84*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /* 89*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /* 94*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                /*99*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /*104*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /*109*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /*114*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /*119*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /*124*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /*129*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /*134*/ NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                                       NULL,
+                               /*139*/ NULL,
+                                       NULL,
+                                       NULL
 };
 
 /*MC
